@@ -1,5 +1,6 @@
 from uuid import uuid1
-
+import time
+from decimal import Decimal
 from polyglotdb.exceptions import GraphModelError
 
 from ..base.helper import key_for_cypher, value_for_cypher
@@ -76,9 +77,13 @@ class LinguisticAnnotation(BaseAnnotation):
         self._subs = {}
         self._speaker = None
         self._discourse = None
+        self._tracks = {}
 
         self._preloaded = False
 
+    def __str__(self):
+        return '<{} annotation with id: {}>'.format(self._type, self._node['id'])
+
     @property
     def corpus_context(self):
         return self._corpus_context
@@ -123,15 +128,82 @@ class LinguisticAnnotation(BaseAnnotation):
         """ Returns sorted untion of node property keys and type_node property keys """
         return sorted(set(self._node.keys()) | set(self._type_node.keys()))
 
+    def _load_track(self, track_attribute):
+        if self._type == 'utterance':
+            utt_id = self.id
+        else:
+            utt_id = self.utterance.id
+        results = track_attribute.hydrate(self.corpus_context, utt_id, self.begin, self.end)
+        if track_attribute.attribute.relative_time:
+            begin = Decimal(self.begin)
+            end = Decimal(self.end)
+            duration = end - begin
+            for p in results:
+                p.time = (p.time - begin) / duration
+
+        self._tracks[track_attribute.attribute.label] = results
+
+    @property
+    def pitch_track(self):
+        if 'pitch' not in self._tracks:
+            self._load_track(getattr(getattr(self.corpus_context, self._type), 'pitch'))
+        data = self._tracks['pitch']
+        return data
+
+    @property
+    def waveform(self):
+        signal, sr = self.corpus_context.load_waveform(self.discourse.name, 'low_freq', begin=self.begin, end=self.end)
+        step = 1 / sr
+        data = [{'amplitude': float(p), 'time': i * step + self.begin} for i, p in enumerate(signal)]
+        return data
+
+    @property
+    def spectrogram(self):
+        orig, time_step, freq_step = self.corpus_context.generate_spectrogram(self.discourse.name, 'consonant',
+                                                                              begin=self.begin,
+                                                                              end=self.end)
+        reshaped = []
+
+        for i in range(orig.shape[0]):
+            for j in range(orig.shape[1]):
+                reshaped.append({'time': j * time_step + self.begin, 'frequency': i * freq_step,
+                                 'power': float(orig[i, j])})
+        data = {'values': reshaped,
+                'time_step': time_step,
+                'freq_step': freq_step,
+                'num_time_bins': orig.shape[1],
+                'num_freq_bins': orig.shape[0]}
+        return data
+
+    @property
+    def spectrogram_fast(self):
+        orig, time_step, freq_step = self.corpus_context.generate_spectrogram(self.discourse.name, 'consonant',
+                                                                              begin=self.begin,
+                                                                              end=self.end)
+        data = {'values': orig,
+                'time_step': time_step,
+                'freq_step': freq_step,
+                'num_time_bins': orig.shape[1],
+                'num_freq_bins': orig.shape[0]}
+        return data
+
     def __getattr__(self, key):
         if self.corpus_context is None:
             raise (GraphModelError('This object is not bound to a corpus context.'))
         if self._id is None:
             raise (GraphModelError('This object has not been loaded with an id yet.'))
+        if key == self._type:
+            return self
+        if key == 'current':
+            return self
+        if key == 'label' and self._type == 'utterance':
+            return '{} ({} to {})'.format(self.discourse.name, self.begin, self.end)
         if key == 'previous':
             if self._previous == 'empty':
                 return None
             if self._previous is None:
+                print('Warning: fetching previous annotation from the database, '
+                      'preload this annotation for faster access.')
                 res = list(self.corpus_context.execute_cypher(
                     '''MATCH (previous_type)<-[:is_a]-(previous_token)-[:precedes]->(token {id: {id}})
                         RETURN previous_token, previous_type''', id=self._id))
@@ -146,6 +218,8 @@ class LinguisticAnnotation(BaseAnnotation):
             if self._following == 'empty':
                 return None
             if self._following is None:
+                print('Warning: fetching following annotation from the database, '
+                      'preload this annotation for faster access.')
                 res = list(self.corpus_context.execute_cypher(
                     '''MATCH (following_type)<-[:is_a]-(following_token)<-[:precedes]-(token {id: {id}})
                             RETURN following_token, following_type''', id=self._id))
@@ -156,10 +230,24 @@ class LinguisticAnnotation(BaseAnnotation):
                 self._following.node = res[0]['following_token']
                 self._following.type_node = res[0]['following_type']
             return self._following
+        if key.startswith('previous'):
+            p, key = key.split('_', 1)
+            p = self.previous
+            if p is None:
+                return None
+            return getattr(p, key)
+        if key.startswith('following'):
+            p, key = key.split('_', 1)
+            f = self.following
+            if f is None:
+                return None
+            return getattr(f, key)
         if key == 'speaker':
             if self._speaker == 'empty':
                 return None
             if self._speaker is None:
+                print('Warning: fetching speaker information from the database, '
+                      'preload speakers for faster access.')
                 res = list(self.corpus_context.execute_cypher(
                     '''MATCH (speaker:Speaker)<-[:spoken_by]-(token {id: {id}})
                         RETURN speaker''', id=self._id))
@@ -173,6 +261,8 @@ class LinguisticAnnotation(BaseAnnotation):
             if self._discourse == 'empty':
                 return None
             if self._discourse is None:
+                print('Warning: fetching discourse information from the database, '
+                      'preload discourses for faster access.')
                 res = list(self.corpus_context.execute_cypher(
                     '''MATCH (discourse:Discourse)<-[:spoken_in]-(token {id: {id}})
                         RETURN discourse''', id=self._id))
@@ -184,6 +274,8 @@ class LinguisticAnnotation(BaseAnnotation):
             return self._discourse
         if key in self.corpus_context.hierarchy.get_lower_types(self._type):
             if key not in self._subs:
+                print('Warning: fetching {0} information from the database, '
+                      'preload {0} annotations for faster access.'.format(key))
                 res = self.corpus_context.execute_cypher(
                     '''MATCH (lower_type)<-[:is_a]-(lower_token:{a_type})-[:contained_by*1..]->(token {{id: {{id}}}})
                         RETURN lower_token, lower_type ORDER BY lower_token.begin'''.format(a_type=key), id=self._id)
@@ -196,6 +288,8 @@ class LinguisticAnnotation(BaseAnnotation):
             return self._subs[key]
         if key in self.corpus_context.hierarchy.get_higher_types(self._type):
             if key not in self._supers:
+                print('Warning: fetching {0} information from the database, '
+                      'preload {0} annotations for faster access.'.format(key))
                 res = list(self.corpus_context.execute_cypher(
                     '''MATCH (higher_type)<-[:is_a]-(higher_token:{a_type})<-[:contained_by*1..]-(token {{id: {{id}}}})
                         RETURN higher_token, higher_type'''.format(a_type=key), id=self._id))
@@ -211,6 +305,8 @@ class LinguisticAnnotation(BaseAnnotation):
                 if self._preloaded and key not in self._subannotations:
                     return []
                 elif key not in self._subannotations:
+                    print('Warning: fetching {0} information from the database, '
+                          'preload {0} annotations for faster access.'.format(key))
                     res = self.corpus_context.execute_cypher(
                         '''MATCH (sub:{a_type})-[:annotates]->(token {{id: {{id}}}})
                             RETURN sub'''.format(a_type=key), id=self._id)
@@ -224,6 +320,8 @@ class LinguisticAnnotation(BaseAnnotation):
                 return self._subannotations[key]
         except KeyError:
             pass
+        if key == 'duration':
+            return self.end - self.begin
         if key in self._node.keys():
             return self._node[key]
         if key in self._type_node.keys():
@@ -240,10 +338,10 @@ class LinguisticAnnotation(BaseAnnotation):
         """
         for k, v in kwargs.items():
             if k in self._type_node.keys():
-                self._type_node.properties.update(**{k: v})
-            self._node.properties.update(**{k: v})
+                self._type_node._update({k: v})
+            self._node._update({k: v})
         if self._node['begin'] > self._node['end']:
-            self._node.properties.update(begin=self._node['end'], end=self._node['begin'])
+            self._node._update({}, begin=self._node['end'], end=self._node['begin'])
         self._unsaved = True
 
     def save(self):
@@ -276,10 +374,13 @@ class LinguisticAnnotation(BaseAnnotation):
 
     @property
     def channel(self):
-        speaker = self.corpus_context.census[self.speaker.name]
-        for x in speaker.discourses:
-            if x.discourse.name == self.discourse.name:
-                return x.channel
+        statement = '''MATCH (s:Speaker:{corpus_name})-[r:speaks_in]->(d:Discourse:{corpus_name})
+        WHERE s.name = {{speaker_name}} and d.name = {{discourse_name}}
+        RETURN r.channel as channel'''.format(corpus_name=self.corpus_context.cypher_safe_name)
+        results = self.corpus_context.execute_cypher(statement, speaker_name=self.speaker.name,
+                                                     discourse_name=self.discourse.name)
+        for r in results:
+            return r['channel']
         return None
 
     @property
@@ -327,30 +428,34 @@ class LinguisticAnnotation(BaseAnnotation):
 
         properties : dict
         """
-        if 'begin' not in properties:
-            properties['begin'] = self.begin
-        if 'end' not in properties:
-            properties['end'] = self.end
+        # if 'begin' not in properties:
+        #    properties['begin'] = self.begin
+        # if 'end' not in properties:
+        #    properties['end'] = self.end
         properties['id'] = str(uuid1())
-
         discourse = self.discourse.name
-
-        self.corpus_context.hierarchy.add_subannotation_type(self._type, type)
-        statement = '''MATCH (n:{type}:{corpus} {{id:{{id}}}})
-        CREATE (n)<-[r:annotates]-(sub:{sub_type}:{corpus} {{{props}}})
+        statement = '''MATCH (n:{type}:{corpus} {{id:{{a_id}}}})
+        CREATE (n)<-[r:annotates]-(sub:{sub_type}:{corpus})
+        WITH sub, r
+        SET {props}
         return sub, r'''
-        props = ['{}: {}'.format(key_for_cypher(k), value_for_cypher(v))
-                 for k, v in properties.items()]
+        props = []
+        for k, v in properties.items():
+            props.append('sub.%s = {%s}' % (k, k))
+            if self._corpus_context.hierarchy.has_subannotation_property(type, k):
+                for name, t in self._corpus_context.hierarchy.subannotation_properties[type]:
+                    if name == k:
+                        properties[k] = t(v)
         statement = statement.format(type=self._type, sub_type=type,
                                      corpus=self.corpus_context.cypher_safe_name, props=', '.join(props))
 
         if transaction is not None:
-            transaction.append(statement, id=self._id)
+            transaction.append(statement, a_id=self._id, **properties)
         else:
             sa = SubAnnotation(self.corpus_context)
             sa._annotation = self
             sa._type = type
-            res = self.corpus_context.execute_cypher(statement, id=self._id).single()
+            res = self.corpus_context.execute_cypher(statement, a_id=self._id, **properties).single()
 
             sa.node = res['sub']
             rel = res['r']
@@ -358,7 +463,7 @@ class LinguisticAnnotation(BaseAnnotation):
             if type not in self._subannotations:
                 self._subannotations[type] = []
             self._subannotations[type].append(sa)
-            self._subannotations[type].sort(key=lambda x: x.begin)
+            # self._subannotations[type].sort(key=lambda x: x.begin)
             return sa.node, rel
 
 
@@ -382,7 +487,9 @@ class SubAnnotation(BaseAnnotation):
             return self._node[key]
         if key == 'label':
             return None
-        raise (AttributeError)
+        if not self._corpus_context.hierarchy.has_subannotation_property(self._type, key):
+            raise AttributeError
+        return None
 
     def update_properties(self, **kwargs):
         """ 
@@ -393,9 +500,9 @@ class SubAnnotation(BaseAnnotation):
         kwards : dict
             keyword arguments to update properties
         """
-        self._node.properties.update(**kwargs)
+        self._node._update(kwargs)
         if self._node['begin'] > self._node['end']:
-            self._node.properties.update(begin=self._node['end'], end=self._node['begin'])
+            self._node._update({}, begin=self._node['end'], end=self._node['begin'])
         self._unsaved = True
 
     @property
@@ -409,7 +516,8 @@ class SubAnnotation(BaseAnnotation):
         self._node = item
         self._id = item['id']
         for x in self._node.labels:
-            if x in self.corpus_context.hierarchy.subannotations[self._annotation._type]:
+            if self._annotation._type in self.corpus_context.hierarchy.subannotations and x in \
+                    self.corpus_context.hierarchy.subannotations[self._annotation._type]:
                 self._type = x
                 break
 
